use crate::store::object::object::Obj;
use std::path::PathBuf;
use crate::store::object::blob::Blob;
use std::iter::{Iter, Empty};
use crate::store::object::object::ObjMethods; 
use std::fs::{self, File, OpenOptions};
use std::io::{self, BufRead, BufReader, Write, Lines};
use crate::utils::read;

pub struct Tree {
    pub obj: Obj,
    pub lines: Option<Lines<BufReader<File>>>,
    pub lines_iter: Option<Iter<String, Lines<BufReader<File>>>>,

}


impl Tree {

    pub fn from_head() -> Self {
        Tree {
            obj: Obj::from_head(),
            lines: None,
            lines_iter: None
        }
    }

    pub fn from_path(path: PathBuf) -> Self {
        Tree {
            obj: Obj::from_path(path),
            lines: None,
            lines_iter: None
        }
    }

    pub fn read_data(&mut self) {
        if self.lines.is_none() {
            if let Ok(mut file) = File::open(self.get_obj_path()) {
                self.lines = Some(BufReader::new(file).lines());
                self.lines_iter = Some(self.lines.iter());
            }
        }
    }

    fn get_iter(&mut self) -> Iter<String, Lines<BufReader<File>>> {
        if let Some(iter) = self.lines_iter {
            iter
        }
        std::process::exit(4);
    }

    pub fn next(&mut self) -> Option<Obj> {
        self.read_data();
        let iter = self.get_iter();
        match iter.next() {
            Some(line) => Some(Obj::from_line(line)),
            None => None
        }
    }


    // create
}


//pub fn add(path: PathBuf, date: &str, up_parent: bool) -> io::Result<()> {
//    let (line, hash, name) = parse_path(path.clone(), false);
//
//    // add tree reference to parent
//    if path.iter().count() == 1 {
//        head::add_line(line)?;
//    } else {
//        add_node(path.parent().unwrap(), &line)?;
//    }
//
//    // create tree object
//    let mut content = name;
//    content.push_str(" ");
//    content.push_str(date);
//    create_obj(hash, &content)?;
//
//    // update date for all parent
//    if up_parent {
//        update_dates(path, date)?;
//    }
//
//    Ok(())
//}
//
//pub fn rm(path: PathBuf) -> io::Result<()> {
//    let (_, lines) = read(path_buf_to_string(path.to_path_buf())).unwrap();
//    for line in lines {
//        let (ftype, hash, _) = parse_line(line.unwrap());
//        if ftype == String::from("blob") {
//            object::rm(&hash)?;
//        } else {
//            rm_hash(hash)?;
//        }
//    }
//    Ok(())
//}
//
//fn rm_hash(hash: String) -> io::Result<()> {
//    let mut obj_p = path::objects();
//    let (dir, res) = hash.split_at(2);
//    obj_p.push(dir);
//    obj_p.push(res);
//
//    match read::read_lines(obj_p) {
//        Ok(mut reader) => {
//            reader.next();
//            for line in reader {
//                let (ftype, hash, _) = parse_line(line.unwrap());
//                if ftype == String::from("blob") {
//                    object::rm(&hash)?;
//                } else {
//                    rm_hash(hash)?;
//                }
//            }
//        },
//        Err(err) => {
//            eprintln!("error reading tree: {}", err);
//        },
//    }
//    Ok(())
//}
//
//pub fn read(tree: String) -> Option<(String, io::Lines<io::BufReader<File>>)> {
//    let mut obj_p = path::objects();
//
//    let (dir, res) = hash_obj(&tree);
//    obj_p.push(dir);
//    obj_p.push(res);
//    
//    match read::read_lines(obj_p) {
//        Ok(mut reader) => {
//            let name = match reader.next() {
//                Some(Ok(line)) => line,
//                _ => String::new(),
//            };
//            Some((name, reader))
//        },
//        Err(err) => {
//            eprintln!("error reading tree: {}", err);
//            None
//        },
//    }
//}
//
//pub fn parse_line(line: String) -> (String, String, String) {
//    let mut split = line.rsplit(' ');
//    if split.clone().count() != 3 {
//        eprintln!("fatal: invalid object(s)");
//        std::process::exit(1);
//    }
//
//    let name = split.next().unwrap();
//    let hash = split.next().unwrap();
//    let ftype = split.next().unwrap();
//    (String::from(ftype), String::from(hash), String::from(name))
//}
